#!/usr/bin/env python

#   Xnt -- A Wrapper Build Tool
#   Copyright (C) 2012  Kenny Ballou

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.

#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import subprocess
import time
import shutil
import zipfile
import contextlib
import glob
import logging

logger = logging.getLogger(__name__)

#File associated tasks
def expandpath(path):
    """
    Expand path using globs to a possibly empty list of directories
    """
    return glob.iglob(path)

def cp(src="",dst="",files=[]):
    assert dst and src or len(files) > 0
    logger.info("Copying %s to %s", src, dst)
    def copy(s,d):
        if os.path.isdir(s):
            shutil.copytree(s,d)
        else:
            shutil.copy2(s,d)
    if src:
        copy(src, dst)
    else:
        for f in files:
            copy(f, dst)

def mv(src,dst):
    logger.info("Moving %s to %s", src, dst)
    shutil.move(src,dst)

def mkdir(dir,mode=0o777):
    if os.path.exists(dir):
        return
    logger.info("Making directory %s with mode %o", dir, mode)
    try:
        os.mkdir(dir,mode)
    except IOError as e:
        log(e, logging.WARNING)
    except:
        raise

def rm(*fileset):
    try:
        for g in fileset:
            for f in expandpath(g):
                if not os.path.exists(f):
                    continue
                logger.info("Removing %s", f)
                if os.path.isdir(f):
                    shutil.rmtree(f)
                else:
                    os.remove(f)
    except OSError as e:
        log(e, logging.WARNING)
    except:
        raise

def zip(dir,zipfilename):
    logger.info("Zipping %s as %s", dir, zipfilename)
    assert os.path.isdir(dir) and zipfilename
    with contextlib.closing(zipfile.ZipFile(
        zipfilename,
        "w",
        zipfile.ZIP_DEFLATED)) as z:
        for root, dirs, files in os.walk(dir):
            for fn in files:
                absfn = os.path.join(root, fn)
                zfn = absfn[len(dir)+len(os.sep):]
                z.write(absfn, zfn)

#Misc Tasks
def echo(msg, tofile):
    with open(tofile, "w") as f:
        f.write(msg)

def log(msg="",lvl=logging.INFO):
    logger.log(lvl, msg)

def xnt(target, path):
    """
    Invoke xnt on another build file in a different directory
    """
    import xnt.xenant
    xnt.xenant.invokeBuild(
        xnt.xenant.__loadBuild(path),
        target)

def call(command, stdout=None, stderr=None):
    """
    Execute the given command, redirecting stdout and stderr
    to optionally given files
    param: command - list of command and arguments
    param: stdout - file to redirect standard output to, if given
    param: stderr - file to redirect standard error to, if given
    """
    return subprocess.call(args=command, stdout=stdout, stderr=stderr)

def setup(commands, dir=""):
    """
    Invoke the ``setup.py`` file in the current or specified directory
    param: commands - list of commands and options to run/ append
    param: dir - (optional) directory to run from
    """
    cmd = [sys.executable, "setup.py",]
    for c in commands:
        cmd.append(c)
    cwd = os.getcwd()
    if dir:
        os.chdir(dir)
    ec = call(cmd)
    os.chdir(cwd)
    return ec



#!/usr/bin/env python

#   Xnt -- A Wrapper Build Tool
#   Copyright (C) 2012  Kenny Ballou

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.

#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import os
import shutil
import xnt.tasks
import unittest


class TaskMiscTests(unittest.TestCase):
    def setUp(self):
        os.mkdir("temp")
        os.mkdir("temp/testfolder1")
        for i in range(1, 5):
            with open("temp/testfile" + str(i), "w") as f:
                f.write("this is a test file")
        with open("temp/test.py", "w") as test:
            test.write("#!/usr/bin/env python\n")
            test.write("import sys\n")
            test.write("sys.stdout.write(sys.argv[1])\n")
            test.write("sys.stderr.write(sys.argv[2])\n")

    def tearDown(self):
        shutil.rmtree("temp")

    def test_echo(self):
        xnt.tasks.echo("this is my cool echo", "temp/mynewcoolfile")
        self.assertTrue(os.path.exists("temp/mynewcoolfile"))
        with open("temp/mynewcoolfile", "r") as f:
            self.assertEqual("this is my cool echo", f.read())

    def test_call(self):
        out = open("temp/testout", "w")
        err = open("temp/testerr", "w")
        xnt.tasks.call(["python",
                        os.path.abspath("temp/test.py"),
                        "42", "hello"],
                       out,
                       err)
        out.close()
        err.close()
        self.assertTrue(os.path.exists("temp/testout"))
        self.assertTrue(os.path.exists("temp/testerr"))
        with open("temp/testout", "r") as o:
            self.assertEqual("42", o.read())
        with open("temp/testerr", "r") as e:
            self.assertEqual("hello", e.read())

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

#   Xnt -- A Wrapper Build Tool
#   Copyright (C) 2012  Kenny Ballou

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.

#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import time
import logging
from xnt.cmdoptions import options
from xnt.commands import commands
from xnt.commands.target import TargetCommand

logging.basicConfig(format="%(asctime)s:%(levelname)s:%(message)s")
logger = logging.Logger(name=__name__)
logger.addHandler(logging.StreamHandler())

def main():
    start_time = time.time()
    params = list(p for p in sys.argv[1:] if p.startswith('-D'))
    flags = list(o for o in sys.argv[1:]
        if o.startswith('-') and o not in params)
    cmds = list(c for c in sys.argv[1:]
        if c not in flags and c not in params)
    #Loop flags and apply them
    for flag in flags:
        if flag in options:
            options[flag]()
        else:
            logger.debug("%s is not a vaild option", flag)
    #run things
    cmd_found = False
    for cmd in cmds:
        if cmd in commands:
            cmd_found = True
            if commands[cmd].needs_build:
                command = commands[cmd](loadBuild())
            else:
                command = commands[cmd]()
            ec = command.run()
    if cmd_found == False:
        command = TargetCommand(loadBuild())
        ec = command.run(targets=cmds, props=params)
    elapsed_time = time.time() - start_time
    logger.info("Execution time: %.3f", elapsed_time)
    if ec != 0:
        logger.info("Failure")
    from xnt.tasks import rm
    rm("build.pyc",
       "__pycache__")
    if ec != 0:
        sys.exit(ec)

def loadBuild(path=""):
    if not path:
        path = os.getcwd()
    else:
        path = os.path.abspath(path)
    sys.path.append(path)
    cwd = os.getcwd()
    os.chdir(path)
    if not os.path.exists("build.py"):
        logger.error("There was no build file")
        sys.exit(1)
    try:
        return __import__("build", fromlist=[])
    except ImportError:
        logger.error("HOW?!")
        return None
    finally:
        sys.path.remove(path)
        del sys.modules["build"]
        os.chdir(cwd)

if __name__ == "__main__":
    ec = main()
    if ec:
        sys.exit(ec)



#!/usr/bin/env python

#   Xnt -- A Wrapper Build Tool
#   Copyright (C) 2012  Kenny Ballou

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.

#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

from xnt.commands.help import HelpCommand
from xnt.commands.listtargets import ListTargetsCommand
from xnt.commands.version import VersionCommand
from xnt.commands.target import TargetCommand

commands = {
    HelpCommand.name: HelpCommand,
    ListTargetsCommand.name: ListTargetsCommand,
    VersionCommand.name: VersionCommand,
    TargetCommand.name: TargetCommand,
}

def get_summaries(ignore_hidden=True):
    items = []

    for name, command_class in commands.items():
        if ignore_hidden and command_class.hidden:
            continue
        items.append((name, command_class.summary))

    return sorted(items)



#!/usr/bin/env python

#   Xnt -- A Wrapper Build Tool
#   Copyright (C) 2012  Kenny Ballou

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.

#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Common Compilers
"""

import os
import logging
import sys
from xnt.tasks import call

logger = logging.getLogger(__name__)

def gcc(src, flags=[]):
    """gcc compiler, non-named output file"""
    return _gcc(src, flags)

def gpp(src, flags=[]):
    """g++ compiler, non-named output file"""
    return _gcc(src, flags, "g++")

def gcc_o(src, o, flags=[]):
    """gcc compiler, with output file"""
    return _gcc_o(src, o, flags)

def gpp_o(src, o, flags=[]):
    """g++ compiler, with output file"""
    return _gcc_o(src, o, flags, "g++")

def javac(src, flags=[]):
    """Javac: compile Java programs"""
    logger.info("Compiling %s", src)
    cmd = __generateCommand(src, flags, "javac")
    return __compile(cmd)

def _gcc(src, flags=[], compiler="gcc"):
    logger.info("Compiling %s", src)
    return __compile(__generateCommand(src, flags, compiler))

def _gcc_o(src, o, flags=[], compiler="gcc"):
    logger.info("Compiling %s to %s", src, o)
    cmd = __generateCommand(src, flags, compiler)
    cmd.append("-o")
    cmd.append(o)
    return __compile(cmd)

def __generateCommand(src, flags=[], compiler="gcc"):
    cmd = [compiler, src]
    for f in flags:
        cmd.append(f)
    return cmd

def __compile(cmd):
    return call(cmd)

def __is_newer(a, b):
    return os.path.getmtime(a) > os.path.getmtime(b)



#!/usr/bin/env python

#   Xnt -- A Wrapper Build Tool
#   Copyright (C) 2012  Kenny Ballou

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.

#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

__version__ = "Xnt 0.5.0"
__license__ = """
   Xnt -- A Wrapper Build Tool
   Copyright (C) 2012  Kenny Ballou

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from xnt.tasks import *

def target(fn):
    def wrap():
        print(fn.__name__ + ":")
        return fn()
    wrap.decorator = "target"
    wrap.__doc__ = fn.__doc__
    return wrap



#!/usr/bin/env python

#   Xnt -- A Wrapper Build Tool
#   Copyright (C) 2012  Kenny Ballou

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.

#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import os
import shutil
import xnt.build.cc as cc
import unittest

#http://stackoverflow.com/questions/377017/test-if-executable-exists-in-python
def which(program):
    def is_exe(fpath):
        return os.path.isfile(fpath) and os.access(fpath, os.X_OK)

    fpath, fname = os.path.split(program)
    if fpath:
        if is_exe(program):
            return program
    else:
        for path in os.environ["PATH"].split(os.pathsep):
            path = path.strip('"')
            exe_file = os.path.join(path, program)
            if is_exe(exe_file):
                return exe_file

    return None

@unittest.skipUnless(which("gcc"), "gcc is not in your path")
class gccTests(unittest.TestCase):
    def setUp(self):
        os.mkdir("temp")
        with open("temp/hello.c", "w") as f:
            f.write("""
            #include <stdio.h>
            int main() {
                printf("Hello, World!\\n");
                return 0;
            }
            """)

    def tearDown(self):
        shutil.rmtree("temp")

    def test_gcc(self):
        oldPath = os.getcwd()
        os.chdir("temp")
        cc.gcc("hello.c")
        self.assertTrue(os.path.isfile("a.out"))
        os.chdir(oldPath)

    def test_gcc_o(self):
        cc.gcc_o("temp/hello.c", "temp/hello")
        self.assertTrue(os.path.isfile("temp/hello"))

@unittest.skipUnless(which("g++"), "g++ is not in your path")
class gppTests(unittest.TestCase):
    def setUp(self):
        os.mkdir("temp")
        with open("temp/hello.cpp", "w") as f:
            f.write("""
            #include <iostream>
            int main() {
                std::cout << "Hello, World!" << std::endl;
                return 0;
            }
            """)

    def tearDown(self):
        shutil.rmtree("temp")

    def test_gpp(self):
        oldPath = os.getcwd()
        os.chdir("temp")
        cc.gpp("hello.cpp")
        self.assertTrue("a.out")
        os.chdir(oldPath)

    def test_gpp_o(self):
        cc.gpp_o("temp/hello.cpp", "temp/hello")
        self.assertTrue(os.path.isfile("temp/hello"))

@unittest.skipUnless(which("javac"), "javac is not in your path")
class javacTests(unittest.TestCase):
    def setUp(self):
        os.mkdir("temp")
        with open("temp/hello.java", "w") as f:
            f.write("""
            class hello {
                public static void main(String[] args) {
                    System.out.println("Hello, World!");
                }
            }
            """)

    def tearDown(self):
        shutil.rmtree("temp")

    def test_javac(self):
        oldPath = os.getcwd()
        os.chdir("temp")
        cc.javac("hello.java")
        self.assertTrue(os.path.isfile("hello.class"))
        os.chdir(oldPath)

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

#   Xnt -- A Wrapper Build Tool
#   Copyright (C) 2012  Kenny Ballou

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.

#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import os
import shutil
import xnt.tasks
import unittest


class TaskRemoveTests(unittest.TestCase):
    def setUp(self):
        os.mkdir("temp")
        os.mkdir("temp/testfolder1")
        for i in range(1, 5):
            with open("temp/testfile" + str(i), "w") as f:
                f.write("this is a test file")

    def tearDown(self):
        shutil.rmtree("temp")

    def test_mkdir(self):
        xnt.tasks.mkdir("temp/mynewtestfolder")
        self.assertTrue(os.path.exists("temp/mynewtestfolder"))
        self.assertTrue(os.path.exists("temp/testfolder1"))
        xnt.tasks.mkdir("temp/testfolder1")

    def test_rm(self):
        xnt.tasks.rm("temp/testfolder1")
        self.assertFalse(os.path.exists("temp/testfolder1"))
        xnt.tasks.rm("temp/testfile1")


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

#   Xnt -- A Wrapper Build Tool
#   Copyright (C) 2012  Kenny Ballou

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.

#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import shutil
from setuptools import setup, find_packages

def read(fname):
    return "\n" + open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name="Xnt",
    version="0.5.0",
    author="Kenny Ballou",
    author_email="kennethmgballou@gmail.com",
    url="https://github.com/devnulltao/Xnt",
    description=("High-Level build script for doing more complex build tasks"),
    packages=find_packages(),
    test_suite="xnt.tests",
    scripts=["xnt/xenant.py",],
    package_data={
    },
    long_description=read("README"),
    platforms=["Linux", "Windows",],
    entry_points={
        'console_scripts': [
            'xnt = xnt.xenant:main',
        ],
    },
    install_requires=['distribute',],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Operating System :: Microsoft :: Windows :: Windows 7',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Utilities',
    ],
)



#!/usr/bin/env python

#   Xnt -- A Wrapper Build Tool
#   Copyright (C) 2012  Kenny Ballou

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.

#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

from xnt.basecommand import Command
from xnt.status_codes import SUCCESS, ERROR, UNKNOWN_ERROR
import logging

logger = logging.getLogger("xnt")

class TargetCommand(Command):
    name = '<target>'
    usage = """"""
    summary = "Invokes target(s) in build.py"
    needs_build = True

    def __init__(self, build):
        self.build = build

    def run(self, targets=[], props=[]):
        if targets:
            for t in targets:
                ec = self.callTarget(t, props)
                if ec:
                    return ec
            return SUCCESS
        else:
            return self.callTarget("default", props)

    def callTarget(self, targetName, props):
        def processParams(params, buildProperties={}):
            properties = buildProperties if buildProperties is not None else {}
            for p in params:
                name, value = p[2:].split("=")
                properties[name] = value
            return properties
        def __getProperties():
            try:
                return getattr(self.build, "properties")
            except AttributeError:
                return None
        try:
            if len(props) > 0:
                setattr(self.build,
                        "properties",
                        processParams(props, __getProperties()))
            target = getattr(self.build, targetName)
            ec = target()
            return ec if ec else 0
        except AttributeError:
            logger.warning("There was no target: %s", targetName)
            return ERROR
        except Exception as ex:
            logger.error(ex)
            return UNKNOWN_ERROR




#!/usr/bin/env python

#   Xnt -- A Wrapper Build Tool
#   Copyright (C) 2012  Kenny Ballou

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.

#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

from xnt.basecommand import Command
from xnt.status_codes import SUCCESS, ERROR
import logging

logger = logging.getLogger(__name__)

class ListTargetsCommand(Command):
    name = 'list-targets'
    usage = """"""
    summary = "Prints targets in build file"
    needs_build = True

    def __init__(self, build):
        self.build = build

    def run(self, arguments=[]):
        logger.debug("build is null? %s", self.build == None)
        try:
            for f in dir(self.build):
                logger.debug("Attribute %s:", f)
                try:
                    fa = getattr(self.build, f)
                    if fa.decorator == "target":
                        print(f + ":")
                        if fa.__doc__:
                            print(fa.__doc__)
                        print("\n")
                except AttributeError:
                    pass
        except Exception as ex:
            logger.error(ex)
            return ERROR
        return SUCCESS
